@@ -23,6 +23,9 @@
 #include <list.h>
 #include <envar.h>
 
+/* Forward declarations */
+struct obstack;
+
 /* Internationalization support */
 #include <gettext.h>
 #define _(s) gettext(s)
@@ -105,7 +108,7 @@ enum {
 #define AP_PROPAGATE     0x10
 #define AP_REQ_CMP       0x20
 #define AP_BINARY_STRING 0x40
-#define AP_USER_FLAG(n) (0x1000<<(n))
+#define AP_USER_FLAG(n) (0x4000<<(n))
 
 #define ADDITIVITY(val) ((val) & 0x3)
 #define SET_ADDITIVITY(val,a) ((val) = ((val) & ~0x3) | (a))
@@ -116,8 +119,8 @@ enum {
 #define CF_HUNTGROUPS 2
 #define CF_MAX        3
 
-#define AF_LHS(cf) (0x0100<<(2*cf))
-#define AF_RHS(cf) (0x0200<<(2*cf))
+#define AF_LHS(cf) (0x0100<<(cf))
+#define AF_RHS(cf) (0x0800<<(cf))
 
 #define AF_DEFAULT_FLAGS (AF_LHS(0)|AF_LHS(1)|AF_LHS(2)\
                          |AF_RHS(0)|AF_RHS(1)|AF_RHS(2))
@@ -231,7 +234,7 @@ typedef struct radius_req {
         u_char                  vector[AUTH_VECTOR_LEN]; /* Rq authenticator */
         u_char                  *secret;      /* Shared secret */
         VALUE_PAIR              *request;     /* Request pairs */
-        
+
         /* Saved reply values */
         int                     reply_code;   /* Reply code */
         VALUE_PAIR              *reply_pairs; /* Reply pairs */
@@ -386,6 +389,9 @@ int format_string_visual(char *buf, int runlen, char *str, int len);
 char *op_to_str(int op);
 int str_to_op(char *str);
 int xlat_keyword(struct keyword *kw, char *str, int def);
+void obstack_grow_backslash_num(struct obstack *stk, char *text, int len, int base);
+void obstack_grow_backslash(struct obstack *stk, char *text, char **endp);
+
 
 /* cryptpass.c */
 void encrypt_password(VALUE_PAIR *pair, char *password,
@@ -407,6 +413,12 @@ struct passwd *rad_getpwnam_r(const char *name, struct passwd *result,
 struct group *rad_getgrnam(const char *name);
 
 /* client.c */
+#define RADCLT_ID            0x1
+#define RADCLT_AUTHENTICATOR 0x2
+
+RADIUS_REQ *rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
+			  VALUE_PAIR *pairlist, int flags, int *authid,
+			  u_char *authvec);
 RADIUS_REQ *rad_clt_send(RADIUS_SERVER_QUEUE *config, int port_type, int code,
 			 VALUE_PAIR *pairlist);
 unsigned rad_clt_message_id(RADIUS_SERVER *server);
